@@ -5,53 +5,35 @@ from ironic_inspector_client.common import http as inspector_http
 import urwid
 
 from tripleodash import clients
-from tripleodash.sections.base import DashboardWidget
+from tripleodash.sections.base import DashboardSection
 from tripleodash import util
 
 DEPLOYED_STATUSES = set(['CREATE_COMPLETE', 'UPDATE_COMPLETE'])
 FAILED_STATUSES = set(['CREATE_FAILED', 'UPDATE_FAILED'])
 
 
-class StackRow(urwid.WidgetWrap):
-    def __init__(self, stack_name, stack_status, creation_time, updated_time,
-                 widget=urwid.Text, selectable=True):
+class OverviewWidget(DashboardSection):
 
-        self._selectable = selectable
-
-        cols = urwid.Columns([
-            (20, widget(str(stack_name))),
-            (20, widget(str(stack_status))),
-            (20, widget(str(creation_time))),
-            (20, widget(str(updated_time))),
-        ])
-
-        super(StackRow, self).__init__(urwid.AttrMap(cols, None, 'reversed'))
-
-    def selectable(self):
-        return self._selectable
-
-    def keypress(self, size, key):
-        return key
-
-
-class OverviewWidget(DashboardWidget):
-
-    def __init__(self):
-        self.heat = clients.heatclient()
-        self.ironic = clients.ironicclient()
-        self.inspector = clients.inspectorclient()
-        self.glance = clients.glanceclient()
+    def __init__(self, clients):
+        super(OverviewWidget, self).__init__(clients, "Overview")
 
     def _images_summary(self):
-        images = list(self.glance.images.list())
+        images = list(self.clients.glance.images.list())
 
         widgets = [
             util.header("Glance Images"),
-            urwid.Text("{0} images found.".format(len(images))),
+            urwid.Text("{0} images uploaded.".format(len(images))),
         ]
 
-        for image in images:
-            widgets.append(urwid.Text("- {0}".format(image.name)))
+        if len(images):
+            for image in images:
+                widgets.append(urwid.Text("- {0}".format(image.name)))
+        else:
+            widgets.extend([
+                urwid.Text("Use these commands to build and upload images:"),
+                urwid.Text("    openstack overcloud image build --all"),
+                urwid.Text("    openstack overcloud image upload"),
+            ])
 
         widgets.append(urwid.Divider())
 
@@ -59,7 +41,7 @@ class OverviewWidget(DashboardWidget):
 
     def _ironic_summary(self):
 
-        nodes = list(self.ironic.node.list())
+        nodes = list(self.clients.ironic.node.list())
         by_provision_state = collections.defaultdict(list)
 
         for node in nodes:
@@ -67,14 +49,22 @@ class OverviewWidget(DashboardWidget):
 
         lines = [
             util.header("Ironic Nodes"),
-            urwid.Text("{0} nodes registered".format(len(nodes))),
+            urwid.Text("{0} nodes registered.".format(len(nodes))),
         ]
 
-        for state, nodes in by_provision_state.iteritems():
-            lines.append(
-                urwid.Text("{0} nodes with the provisioning state '{1}'"
-                           .format(len(nodes), state))
-            )
+        if len(by_provision_state):
+            for state, nodes in by_provision_state.items():
+                lines.append(
+                    urwid.Text("{0} nodes with the provisioning state '{1}'"
+                               .format(len(nodes), state))
+                )
+        else:
+            lines.extend([
+                urwid.Text("Use these commands to register nodes:"),
+                urwid.Text("    openstack baremetal import --json "
+                           "instackenv.json"),
+                urwid.Text("    openstack baremetal configure boot"),
+            ])
 
         lines.append(urwid.Divider())
 
@@ -82,54 +72,85 @@ class OverviewWidget(DashboardWidget):
 
     def _inspector_summary(self):
 
-        nodes = list(self.ironic.node.list())
+        nodes = list(self.clients.ironic.node.list())
         by_introspection_status = collections.defaultdict(list)
 
         for node in nodes:
             try:
-                inspector_status = self.inspector.get_status(node.uuid)
+                inspector_status = self.clients.inspector.get_status(node.uuid)
+                inspector_status = inspector_status['finished']
             except inspector_http.ClientError:
-                inspector_status = {'finished': "Not started"}
-            by_introspection_status[inspector_status['finished']].append(node)
+                inspector_status = "Not started"
+            except clients.ServiceNotAvailable:
+                inspector_status = 'Unknown'
+            by_introspection_status[inspector_status].append(node)
 
-        return [
-            util.header("Node Introspection"),
-            urwid.Text("{0} nodes currently being introspected".format(
-                len(by_introspection_status[False]))),
-            urwid.Text("{0} nodes finished introspection".format(
-                len(by_introspection_status[True]))),
-            urwid.Divider(),
+        lines = [
+            util.header("Node Introspection")
         ]
 
+        if len(by_introspection_status[False]):
+            lines.append(
+                urwid.Text("{0} nodes currently being introspected".format(
+                    len(by_introspection_status[False])))
+            )
+
+        if len(by_introspection_status[True]):
+            lines.append(
+                urwid.Text("{0} nodes finished introspection".format(
+                    len(by_introspection_status[True])))
+            )
+
+        if len(by_introspection_status["Not started"]):
+            lines.append(
+                urwid.Text("{0} nodes not yet started introspection".format(
+                    len(by_introspection_status["Not started"])))
+            )
+
+        if len(by_introspection_status["Unknown"]):
+            lines.append(
+                urwid.Text("Failed to get introspection status for {0} nodes"
+                           .format(len(by_introspection_status["Unknown"])))
+            )
+
+        if len(lines) == 1:
+            return []
+
+        lines.append(urwid.Divider())
+        return lines
+
     def _stack_event_summary(self, stack):
 
-        events = event_utils.get_events(self.heat,
+        events = event_utils.get_events(self.clients.heat,
                                         stack_id=stack.stack_name,
-                                        nested_depth=2,
+                                        nested_depth=1,
                                         event_args={'sort_dir': 'asc'})
 
         return util.heat_event_log_formatter(events[-25:])
 
     def _stacks_summary(self, stacks):
 
-        lines = [
-            StackRow("Stack Name", "Stack Status", "Created Date",
-                     "Updated Date", widget=util.header, selectable=False),
-            urwid.Divider(),
-        ]
+        rows = [(
+            "Stack Name", "Stack Status", "Created Date",
+            "Updated Date"
+        )]
 
         for stack in stacks:
-            lines.append(StackRow(stack.stack_name, stack.stack_status,
-                                  stack.creation_time, stack.updated_time))
+            rows.append((
+                stack.stack_name, stack.stack_status, stack.creation_time,
+                stack.updated_time
+            ))
+
+        lines = list(util.AutoTable(rows).wrapped_widgets())
 
-        lines.append(urwid.Divider())
         lines.append(urwid.Divider())
 
         return lines
 
     def _resource_error(self, stack):
 
-        resources = self.heat.resources.list(stack.stack_name, nested_depth=5)
+        resources = self.clients.heat.resources.list(stack.stack_name,
+                                                     nested_depth=2)
 
         failed = (resource for resource in resources
                   if resource.resource_status in FAILED_STATUSES)
@@ -145,14 +166,15 @@ class OverviewWidget(DashboardWidget):
 
         self.title += "- Not Yet Deployed"
 
-        lines = [
-            util.header("Heat Stack"),
-            urwid.Text("No stacks deployed.", ),
-            urwid.Divider(),
-        ]
+        lines = []
         lines.extend(self._images_summary())
         lines.extend(self._ironic_summary())
         lines.extend(self._inspector_summary())
+        lines.extend([
+            util.header("Heat Stack"),
+            urwid.Text("No stacks deployed.", ),
+            urwid.Divider(),
+        ])
         return lines
 
     def deployed(self, stacks):
@@ -160,10 +182,10 @@ class OverviewWidget(DashboardWidget):
         self.title += "- Deploy Complete"
 
         lines = []
-        lines.extend(self._stacks_summary(stacks))
         lines.extend(self._images_summary())
         lines.extend(self._ironic_summary())
         lines.extend(self._inspector_summary())
+        lines.extend(self._stacks_summary(stacks))
         return lines
 
     def deploying(self, stacks):
@@ -204,7 +226,7 @@ class OverviewWidget(DashboardWidget):
 
         self.title = "Overview "
 
-        stacks = list(self.heat.stacks.list())
+        stacks = list(self.clients.heat.stacks.list())
 
         deployment_exists = len(stacks) > 0
         stack_statuses = set(stack.stack_status for stack in stacks)
